@@ -13,6 +13,7 @@ from app_utils.datetime import (
 )
 from app_utils.urls import reverse_absolute, static_file_absolute_url
 
+from .. import constants
 from ..app_settings import (
     STRUCTURES_DEVELOPER_MODE,
     STRUCTURES_NOTIFICATION_SHOW_MOON_ORE,
@@ -86,7 +87,7 @@ class NotificationBaseEmbed:
         footer_text = "Eve Online"
         if STRUCTURES_DEVELOPER_MODE:
             footer_text = f"{footer_text} ID {self.notification.notification_id}"
-        footer_icon_url = static_file_absolute_url("structures/eve_symbol_128.png")
+        footer_icon_url = static_file_absolute_url("structures/img/eve_symbol_128.png")
         footer = dhooks_lite.Footer(text=footer_text, icon_url=footer_icon_url)
         return dhooks_lite.Embed(
             author=author,
@@ -325,8 +326,8 @@ class NotificationStructureServicesOffline(NotificationStructureEmbed):
         super().__init__(notification)
         self._title = gettext("Structure services off-line")
         self._description += gettext("has all services off-lined.")
-        if self._structure and self._structure.structureservice_set.count() > 0:
-            qs = self._structure.structureservice_set.all().order_by("name")
+        if self._structure and self._structure.services.count() > 0:
+            qs = self._structure.services.all().order_by("name")
             services_list = "\n".join([x.name for x in qs])
             self._description += "\n*{}*".format(services_list)
         self._color = self.COLOR_DANGER
@@ -713,7 +714,7 @@ class NotificationOrbitalEmbed(NotificationBaseEmbed):
             self._parsed_text["planetID"]
         )
         self._structure_type, _ = EveType.objects.get_or_create_esi(
-            EveType.EVE_TYPE_ID_POCO
+            constants.EVE_TYPE_ID_POCO
         )
         solar_system, _ = EveSolarSystem.objects.get_or_create_esi(
             self._parsed_text["solarSystemID"]
@@ -862,7 +863,7 @@ class NotificationSovEmbed(NotificationBaseEmbed):
                 self._parsed_text["campaignEventType"]
             )
         else:
-            structure_type_id = EveType.EVE_TYPE_ID_TCU
+            structure_type_id = constants.EVE_TYPE_ID_TCU
         structure_type, _ = EveType.objects.get_or_create_esi(structure_type_id)
         self._structure_type_name = structure_type.name_localized
         self._sov_owner_link = self._gen_alliance_link(notification.sender.name)
