@@ -11,7 +11,7 @@ from allianceauth.eveonline.models import EveCorporationInfo
 from allianceauth.services.hooks import get_extension_logger
 from app_utils.logging import LoggerAddTag
 
-from .. import __title__
+from .. import __title__, constants
 from ..managers import EveSovereigntyMapManager, EveUniverseManager
 
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
@@ -280,18 +280,13 @@ class EveUniverse(EsiNameLocalization, models.Model):
 class EveCategory(EveUniverse):
     """category in Eve Online"""
 
-    # named category IDs
-    EVE_CATEGORY_ID_ORBITAL = 46
-    EVE_CATEGORY_ID_STARBASE = 23
-    EVE_CATEGORY_ID_STRUCTURE = 65
-
     @property
     def is_starbase(self):
-        return self.id == self.EVE_CATEGORY_ID_STARBASE
+        return self.id == constants.EVE_CATEGORY_ID_STARBASE
 
     @property
     def is_upwell_structure(self):
-        return self.id == self.EVE_CATEGORY_ID_STRUCTURE
+        return self.id == constants.EVE_CATEGORY_ID_STRUCTURE
 
     class EveUniverseMeta:
         esi_pk = "category_id"
@@ -301,12 +296,13 @@ class EveCategory(EveUniverse):
 class EveGroup(EveUniverse):
     """group in Eve Online"""
 
-    EVE_GROUP_ID_CONTROL_TOWER = 365
-    EVE_GROUP_ID_FUEL_BLOCK = 1136
-    EVE_GROUP_ID_REFINERY = 1406
-
     eve_category = models.ForeignKey(
-        EveCategory, on_delete=models.SET_DEFAULT, null=True, default=None, blank=True
+        EveCategory,
+        on_delete=models.SET_DEFAULT,
+        null=True,
+        default=None,
+        blank=True,
+        related_name="eve_groups",
     )
 
     class EveUniverseMeta:
@@ -317,20 +313,17 @@ class EveGroup(EveUniverse):
 class EveType(EveUniverse):
     """type in Eve Online"""
 
-    # named type IDs
-    EVE_TYPE_ID_POCO = 2233
-    EVE_TYPE_ID_TCU = 32226
-    EVE_TYPE_ID_IHUB = 32458
-    EVE_TYPE_ID_STRONTIUM = 16275
-
     # starbase sizes
     STARBASE_SMALL = 1
     STARBASE_MEDIUM = 2
     STARBASE_LARGE = 3
 
     EVE_IMAGESERVER_BASE_URL = "https://images.evetech.net"
+    URL_PROFILE_TYPE = "https://www.kalkoken.org/apps/eveitems/"
 
-    eve_group = models.ForeignKey(EveGroup, on_delete=models.CASCADE)
+    eve_group = models.ForeignKey(
+        EveGroup, on_delete=models.CASCADE, related_name="eve_types"
+    )
 
     class EveUniverseMeta:
         esi_pk = "type_id"
@@ -338,17 +331,17 @@ class EveType(EveUniverse):
 
     @property
     def is_poco(self):
-        return self.id == self.EVE_TYPE_ID_POCO
+        return self.id == constants.EVE_TYPE_ID_POCO
 
     @property
     def is_starbase(self):
-        return self.eve_group_id == EveGroup.EVE_GROUP_ID_CONTROL_TOWER
+        return self.eve_group_id == constants.EVE_GROUP_ID_CONTROL_TOWER
 
     @property
     def is_upwell_structure(self):
-        if self.eve_group.eve_category:
-            return self.eve_group.eve_category.is_upwell_structure
-        else:
+        try:
+            return self.eve_group.eve_category_id == constants.EVE_CATEGORY_ID_STRUCTURE
+        except AttributeError:
             logger.warning(
                 'Group "%s" does not have a category. This is a data error. '
                 "Please update your local SDE data",
@@ -358,7 +351,7 @@ class EveType(EveUniverse):
 
     @property
     def is_fuel_block(self):
-        return self.eve_group_id == EveGroup.EVE_GROUP_ID_FUEL_BLOCK
+        return self.eve_group_id == constants.EVE_GROUP_ID_FUEL_BLOCK
 
     @property
     def starbase_size(self):
@@ -387,6 +380,10 @@ class EveType(EveUniverse):
         else:
             return None
 
+    @property
+    def profile_url(self) -> str:
+        return f"{self.URL_PROFILE_TYPE}?typeId={self.id}"
+
     @classmethod
     def generic_icon_url(cls, type_id: int, size: int = 64) -> str:
         if size < 32 or size > 1024 or (size % 2 != 0):
@@ -414,7 +411,9 @@ class EveRegion(EveUniverse):
 class EveConstellation(EveUniverse):
     """constellation in Eve Online"""
 
-    eve_region = models.ForeignKey(EveRegion, on_delete=models.CASCADE)
+    eve_region = models.ForeignKey(
+        EveRegion, on_delete=models.CASCADE, related_name="eve_constellations"
+    )
 
     class EveUniverseMeta:
         esi_pk = "constellation_id"
@@ -430,7 +429,9 @@ class EveSolarSystem(EveUniverse):
     TYPE_W_SPACE = "w-space"
     TYPE_UNKNOWN = "unknown"
 
-    eve_constellation = models.ForeignKey(EveConstellation, on_delete=models.CASCADE)
+    eve_constellation = models.ForeignKey(
+        EveConstellation, on_delete=models.CASCADE, related_name="eve_solar_systems"
+    )
     security_status = models.FloatField()
 
     class EveUniverseMeta:
@@ -511,7 +512,9 @@ class EvePlanet(EveUniverse):
     position_z = models.FloatField(
         null=True, default=None, blank=True, help_text="z position in the solar system"
     )
-    eve_solar_system = models.ForeignKey(EveSolarSystem, on_delete=models.CASCADE)
+    eve_solar_system = models.ForeignKey(
+        EveSolarSystem, on_delete=models.CASCADE, related_name="eve_planets"
+    )
     eve_type = models.ForeignKey(EveType, on_delete=models.CASCADE)
 
     def _name_localized_generated(self, language: str) -> str:
@@ -547,7 +550,9 @@ class EveMoon(EveUniverse):
     position_z = models.FloatField(
         null=True, default=None, blank=True, help_text="z position in the solar system"
     )
-    eve_solar_system = models.ForeignKey(EveSolarSystem, on_delete=models.CASCADE)
+    eve_solar_system = models.ForeignKey(
+        EveSolarSystem, on_delete=models.CASCADE, related_name="eve_moons"
+    )
 
     def _name_localized_generated(self, language: str):
         """returns a generated localized moon name for the given language"""
